@@ -1,6 +1,8 @@
 import kaa
-from kaa.command import Commands, commandid, is_enable, norec, norerun
-from kaa import document
+from kaa.command import Commands
+from kaa.command import commandid
+from kaa.command import norec
+from kaa.command import norerun
 
 
 class ApplicationCommands(Commands):
@@ -68,7 +70,6 @@ class ApplicationCommands(Commands):
     @norec
     @norerun
     def show_framelist(self, wnd):
-        docs = []
         frames = list(kaa.app.get_frames())
         titles = list(frame.get_title().replace('&', '&&') for frame in frames)
 
@@ -120,7 +121,7 @@ class ApplicationCommands(Commands):
     def editor_nextwindow(self, wnd):
         splitter = wnd.parent.splitter
         if splitter:
-            wnds = [wnd for s, wnd in splitter.walk() if wnd]
+            wnds = [w for s, w in splitter.walk() if w]
             try:
                 n = wnds.index(wnd) + 1
             except ValueError:
@@ -135,7 +136,7 @@ class ApplicationCommands(Commands):
     def editor_prevwindow(self, wnd):
         splitter = wnd.parent.splitter
         if splitter:
-            wnds = [wnd for s, wnd in splitter.walk() if wnd]
+            wnds = [w for s, w in splitter.walk() if w]
             try:
                 n = wnds.index(wnd) - 1
             except ValueError:
