@@ -1,12 +1,9 @@
 import curses
-import itertools
 
 import kaa
 import kaa.context
 from kaa.cui.wnd import Window
 from kaa.cui import splitter, editor
-from kaa.ui.messagebar import messagebarmode
-from kaa import document
 
 
 class ChildFrame(Window, kaa.context.ContextRoot):
@@ -135,6 +132,8 @@ class MainFrame(Window, kaa.context.ContextRoot):
     def on_console_resized(self):
         """Resize wnds"""
         self.height, self.width = self._cwnd.getmaxyx()
+        self.messagebar.set_rect(
+            0, self.height - self.MESSAGEBAR_HEIGHT, self.width, self.height)
 
         if self.inputline and not self.inputline.closed:
             w, h = self.inputline.getsize()
@@ -148,9 +147,6 @@ class MainFrame(Window, kaa.context.ContextRoot):
         for popup in self.popups:
             popup.on_console_resized()
 
-        self.messagebar.set_rect(
-            0, self.height - self.MESSAGEBAR_HEIGHT, self.width, self.height)
-
     def _get_temporary_frame(self):
         # if kaa has only one frame and the frame has temporary document,
         # then dispose the document and reuse the frame.
@@ -213,9 +209,9 @@ class MainFrame(Window, kaa.context.ContextRoot):
 
     def run_refresh(self):
         if self.need_refresh:
-#            if self.need_redraw:
-#                for child in self.walk_children():
-#                    child._cwnd.touchwin()
+            #            if self.need_redraw:
+            #                for child in self.walk_children():
+            #                    child._cwnd.touchwin()
 
             curses.panel.update_panels()
             curses.doupdate()
