@@ -1,13 +1,11 @@
 import os
 import unicodedata
 import sys
-import locale
 import kaa
 import kaa.log
 from kaa import document, encodingdef
 from kaa.filetype.default import defaultmode
 from kaa import consts
-from kaa import encodingdef
 
 
 class FileStorage:
@@ -123,12 +121,10 @@ class FileStorage:
         return doc
 
     def openfile(self, filename, encoding=None, newline=None, nohist=False,
-                 filemustexists=False):
-        if not nohist:
-            self._save_hist(filename)
+                 filemustexists=False, modecls=None):
 
         fileinfo = self.get_fileinfo(filename, encoding, newline)
-        modecls = select_mode(fileinfo.fullpathname)
+        modecls = modecls if modecls else select_mode(fileinfo.fullpathname)
         modecls.update_fileinfo(fileinfo)
 
         textio = self.get_textio(fileinfo, filemustexists)
@@ -140,6 +136,9 @@ class FileStorage:
                 src = src.replace(fileinfo.nlchars, '\n')
             buf.insert(0, src)
 
+            if not nohist:
+                self._save_hist(filename)
+
         doc = document.Document(buf)
         doc.fileinfo = fileinfo
 
