@@ -1,5 +1,8 @@
 from kaa.keyboard import *
 
+# Todo: Splitting key bind table does not make sense.
+# Put them together.
+
 # application commands
 app_keys = {
     (alt, 'm'): 'app.mainmenu',
@@ -8,6 +11,7 @@ app_keys = {
     f9: 'app.global.prev',
     f10: 'app.global.next',
     (alt, 'w'): 'menu.window',
+    (alt, 'z'): 'tools.suspend',
 }
 
 # default cursor commands
