@@ -7,10 +7,10 @@ from kaa import doc_re
 import kaa
 from kaa import keyboard, editmode
 import kaa.log
-from kaa import highlight
 from kaa import theme
 from kaa import screen
 from kaa import addon
+from kaa import syntax_highlight
 
 
 class SearchOption:
@@ -48,6 +48,8 @@ class SearchOption:
 
 SearchOption.LAST_SEARCH = SearchOption()
 
+DefaultTokenizer = syntax_highlight.Tokenizer()
+
 
 class ModeBase:
     (UNDO_INSERT,
@@ -58,7 +60,6 @@ class ModeBase:
 
     SCREEN_NOWRAP = False
     NO_WRAPINDENT = False
-    SCREEN_BUILD_ENTIRE_ROWS = False
     SHOW_LINENO = False
     SHOW_BLANK_LINE = False
     USE_UNDO = True
@@ -74,10 +75,13 @@ class ModeBase:
 
     closed = False
     theme = None
-    highlight = None
     _check_fileupdate = 0
     _last_autoindent = None
 
+    tokenizer = DefaultTokenizer
+
+    DEFAULT_MENU_MESSAGE = ''
+
     @classmethod
     def update_fileinfo(cls, fileinfo, document=None):
         pass
@@ -101,16 +105,32 @@ class ModeBase:
         self.themes = []
         self.init_themes()
         self.init_addon_themes()
-        self._build_theme()
-        kaa.app.translate_theme(self.theme)
 
         self.tokenizers = []
         self.init_tokenizers()
+
+        self._highlight_done = 0
+        self._highlight_iter = None
+
         self.stylemap = {}
         self.stylenamemap = {}
-        self.highlight = highlight.Highlighter(self.tokenizers)
 
         self.setup()
+        self.setup_addon()
+
+        self._build_theme()
+        kaa.app.translate_theme(self.theme)
+
+    def setup_addon(self):
+        names = []
+        for cls in self.__class__.__mro__:
+            modulename = cls.__module__
+            name = '.'.join((modulename, cls.__name__))
+            names.insert(0, name)
+
+        for name in names:
+            for f in addon.get_addon(name, 'setup'):
+                f(self)
 
     def setup(self):
         pass
@@ -127,26 +147,25 @@ class ModeBase:
         self.commands = None
 
         self.is_availables = None
-        self.highlight.close()
-        self.highlight = None
         self.tokenizers = None
+        self.tokenizer = None
         self.stylemap = None
+        self._highlight_done = 0
+        self._highlight_iter = None
 
     def _build_style_map(self):
         self.stylemap[0] = self.theme.get_style('default')
-        for tokenid in self.highlight.tokenids:
-            assert tokenid not in self.stylemap
-            token = self.highlight.get_token(tokenid)
+        for styleid, token in self.tokenizer.styleid_map.items():
             if token:
-                stylename = token.get_stylename(tokenid)
+                stylename = token.get_stylename(styleid)
                 style = self.theme.get_style(stylename)
 
-                self.stylemap[tokenid] = style
-                self.stylenamemap[style.name] = tokenid
+                self.stylemap[styleid] = style
+                self.stylenamemap[style.name] = styleid
             else:
                 style = self.theme.get_style('default')
-                self.stylemap[tokenid] = style
-                self.stylenamemap[style.name] = tokenid
+                self.stylemap[styleid] = style
+                self.stylenamemap[style.name] = styleid
 
     def on_set_document(self, document):
         self.document = document
@@ -157,8 +176,9 @@ class ModeBase:
 
     def on_document_updated(self, pos, inslen, dellen):
         self._last_autoindent = None
-        if self.highlight:
-            self.highlight.updated(self.document, pos, inslen, dellen)
+        if pos <= self._highlight_done:
+            self._highlight_done = pos
+            self._highlight_iter = None
 
     def on_file_saved(self, fileinfo):
         pass
@@ -189,17 +209,29 @@ class ModeBase:
         name = '.'.join((modulename, self.__class__.__name__))
         return name
 
+    def add_keybinds(self, editmode='input', keys=None):
+        if not keys:
+            return
+        if editmode == 'input':
+            self.keybind.add_keybind(keys)
+        elif editmode == 'command':
+            self.keybind_vi_commandmode.add_keybind(keys)
+        elif editmode == 'visual':
+            self.keybind_vi_visualmode.add_keybind(keys)
+        elif editmode == 'visualline':
+            self.keybind_vi_visuallinewisemode.add_keybind(keys)
+
     def init_addon_keys(self):
         name = self.get_class_name()
         keydef = addon.get_addon(name, 'keybind')
-        for editmode, keys in keydef:
-            if editmode == 'input':
+        for edit_mode, keys in keydef:
+            if edit_mode == 'input':
                 self.keybind.add_keybind(keys)
-            elif editmode == 'command':
+            elif edit_mode == 'command':
                 self.keybind_vi_commandmode.add_keybind(keys)
-            elif editmode == 'visual':
+            elif edit_mode == 'visual':
                 self.keybind_vi_visualmode.add_keybind(keys)
-            elif editmode == 'visualline':
+            elif edit_mode == 'visualline':
                 self.keybind_vi_visuallinewisemode.add_keybind(keys)
 
     def init_commands(self):
@@ -237,9 +269,18 @@ class ModeBase:
         themes = addon.get_addon(name, 'style')
         self.themes.extend(themes)
 
+    def add_theme(self, theme):
+        self.themes.append(theme)
+
     def init_tokenizers(self):
         pass
 
+    def add_command(self, f, command_id=None):
+        if command_id is not None:
+            f.COMMAND_ID = command_id
+
+        self.commands[f.COMMAND_ID] = f
+
     def register_commandobj(self, cmds):
         self.commands.update(cmds.get_commands())
         self.is_availables.update(cmds.get_commands_is_enable())
@@ -325,9 +366,9 @@ class ModeBase:
 
         if kaa.app.macro.is_recording():
             kaa.app.macro.record_string(s, overwrite)
-        if self.highlight:
-            # run highlighter a bit to display changes immediately.
-            self.highlight.update_style(self.document, batch=50)
+
+        # run highlighter a bit to display changes immediately.
+        self.run_tokenizer(batch=50)
 
     def on_commands(self, wnd, commandids, n_repeat=1):
         wnd.set_command_repeat(n_repeat)
@@ -358,6 +399,29 @@ class ModeBase:
         finally:
             wnd.set_command_repeat(1)
 
+        def f():
+            import kaa.cui.frame
+            frames = kaa.cui.frame.MainFrame.childframes
+            frame_panels = [f._panel for f in frames]
+
+            import curses.panel
+            panels = [curses.panel.top_panel()]
+            while True:
+                panel = panels[-1].below()
+                if panel:
+                    panels.append(panel)
+                else:
+                    break
+
+            for panel in panels:
+                if panel in frame_panels:
+                    if panel is not frame_panels[0]:
+                        _trace('Invalid frame order: %s%s' %
+                               (self, commandids,))
+                break
+
+        f()
+
     def on_esc_pressed(self, wnd, event):
         pass
 
@@ -475,25 +539,68 @@ class ModeBase:
         if self.closed:
             return
 
-        ret = self.run_highlight()
+        ret = self.run_tokenizer()
         return ret
 
     def get_line_overlays(self):
         return {}
 
+    def _get_highlight_range(self):
+        return (0, self.document.endpos())
+
     HIGHLIGHTBATCH = 300
 
-    def run_highlight(self):
-        if self.highlight:
-            return self.highlight.update_style(
-                self.document,
-                batch=self.HIGHLIGHTBATCH)
+    def run_tokenizer(self, batch=HIGHLIGHTBATCH):
+        range_start, range_end = self._get_highlight_range()
+
+        if self._highlight_done < range_start:
+            self._highlight_done = range_start
+
+        if not self._highlight_iter:
+            f = max(range_start, self._highlight_done - 1)
+            self._highlight_iter = syntax_highlight.begin_tokenizer(
+                self.document, self.tokenizer, f)
+
+        updatefrom = self.document.endpos()
+        updateto = range_start
+        updated = False
+        finished = False
+
+        for n, (f, t, style) in enumerate(self._highlight_iter):
+            f = max(range_start, f)
+            t = min(range_end, t)
+            if f < t:
+                self.document.styles.setints(f, t, style)
+
+            updated = True
+            updatefrom = min(f, updatefrom)
+            updateto = max(t, updateto)
+
+            if batch and (n > batch):
+                break
+
+            if t >= range_end:
+                finished = True
+                break
+        else:
+            finished = True
+
+        if self.document.endpos() == 0 or updated and (updatefrom != updateto):
+            self.document.style_updated(updatefrom, updateto)
+            self._highlight_done = updateto
+
+        # returns False if finished to terminate idle loop.
+        return not finished
 
     def _split_chars(self, begin, end):
         """split characters by character category."""
 
         s = self.document.gettext(begin, end)
-        for key, chars in itertools.groupby(s, unicodedata.category):
+
+        def get_category(c):
+            return unicodedata.category(c)[0]
+
+        for key, chars in itertools.groupby(s, get_category):
             chars = ''.join(chars)
             end = begin + len(chars)
             yield begin, end, chars, key
@@ -508,6 +615,18 @@ class ModeBase:
     RE_SPLITWORD = doc_re.compile('|'.join((
         RE_WORDCHAR, RE_WHITESPACE, RE_LF, RE_HIRAGANA, RE_KATAKANA)))
 
+# http://unicodebook.readthedocs.org/en/latest/unicode.html
+# Unicode 6.0 has 7 character categories, and each category has subcategories:
+#
+# Letter (L): lowercase (Ll), modifier (Lm), titlecase (Lt), uppercase (Lu), other (Lo)
+# Mark (M): spacing combining (Mc), enclosing (Me), non-spacing (Mn)
+# Number (N): decimal digit (Nd), letter (Nl), other (No)
+# Punctuation (P): connector (Pc), dash (Pd), initial quote (Pi), final quote (Pf), open (Ps), close (Pe), other (Po)
+# Symbol (S): currency (Sc), modifier (Sk), math (Sm), other (So)
+# Separator (Z): line (Zl), paragraph (Zp), space (Zs)
+# Other (C): control (Cc), format (Cf), not assigned (Cn), private use
+# (Co), surrogate (Cs)
+
     def split_word(self, begin, all=False):
         """yield word in the document until line ends"""
 
@@ -528,17 +647,18 @@ class ModeBase:
 
     def get_word_at(self, pos):
         tol = self.document.gettol(pos)
-        ret = None
+        prev = None
         for f, t, s, cg in self.split_word(tol):
-            if t <= pos:
-                if cg != 'Z_WHITESPACE':
-                    ret = (f, t, cg)
-            else:
-                if cg not in {'_LF', 'Z_WHITESPACE'}:
-                    ret = (f, t, cg)
-                break
+            if pos == t:
+                if cg not in {'Z_WHITESPACE', '_LF'}:
+                    prev = (f, t, cg)
+            elif pos < t:
+                if cg not in {'Z_WHITESPACE', '_LF'}:
+                    return (f, t, cg)
+                else:
+                    return prev
 
-        return ret
+        return prev
 
     def get_word_list(self):
         words = collections.OrderedDict()
@@ -633,6 +753,8 @@ class ModeBase:
     def calc_next_indent(self, pos):
         return None
 
+    MAX_AUTO_INDENT = 30
+
     def on_auto_indent(self, wnd):
         pos = wnd.cursor.pos
         f, t = self.get_indent_range(pos)
@@ -652,7 +774,8 @@ class ModeBase:
         if indentcol is None:
             indent = self.document.gettext(f, min(pos, t))
         else:
-            indent = self.build_indent_str(indentcol)
+            indent = self.build_indent_str(
+                min(self.MAX_AUTO_INDENT, indentcol))
 
         self.insert_string(wnd, pos, '\n' + indent, update_cursor=False)
         self.delete_ws(wnd, pos + len(indent) + 1)
