@@ -1,17 +1,16 @@
 import curses
 import code
 import sys
-import traceback
 import time
 from contextlib import contextmanager
 import kaa
 from kaa import document
 from kaa.keyboard import *
-from kaa.filetype.default import defaultmode
 from kaa.filetype.python import pythonmode
-from kaa.command import commandid, Commands, norec, norerun
-from kaa.theme import Theme, Style
-from kaa.ui.dialog import dialogmode
+from kaa.command import commandid
+from kaa.command import norec
+from kaa.command import norerun
+from kaa.theme import Style
 
 pythonconsole_keys = {
     ('\r'): 'pythonconsole.newline',
@@ -76,10 +75,6 @@ class PythonConsoleMode(pythonmode.PythonMode):
         super().init_themes()
         self.themes.append(PythonConsoleThemes)
 
-    def setup(self):
-        super().setup()
-        self.highlight.set_mark('current_script')
-
     def on_set_document(self, document):
         super().on_set_document(document)
         self.interp = KaaInterpreter(document=document)
@@ -98,6 +93,9 @@ class PythonConsoleMode(pythonmode.PythonMode):
         f, t = self.document.marks['current_script']
         wnd.cursor.setpos(f)
 
+    def _get_highlight_range(self):
+        return self.document.marks['current_script']
+
     @commandid('pythonconsole.clear')
     @norec
     @norerun
@@ -196,7 +194,8 @@ class PythonConsoleMode(pythonmode.PythonMode):
                 curses.raw()
 
             if not ret:
-                kaa.app.messagebar.set_message('Execued in {} secs'.format(end-start))
+                kaa.app.messagebar.set_message(
+                    'Execued in {} secs'.format(end - start))
                 wnd.document.undo.clear()
                 if s.strip():
                     hist = kaa.app.config.hist(
